
//////////////////////////////////////////////////////////////////////////////////////////
//											//
// file:	cracklinux.h								//
// version:	0.50									//
// author:	copyright (c) 2006 by Marko Euth <linux@euth.de>, www.euth.de		//
// last change: 09.03.2006								//
// description: This Header File is used by the kernel module cracklinux.ko and		//
//		the library libcracklinux.so. It contents common defines.		//
//		There's no need to include it in applications directly.			//
//											//
// license:	This program is free software; you can redistribute it and/or		//
//		modify it under the terms of the GNU General Public License		//
//		as published by the Free Software Foundation; either version 2		//
//		of the License, or (at your option) any later version.			//
//											//
//		This program is distributed in the hope that it will be useful,		//
//		but WITHOUT ANY WARRANTY; without even the implied warranty of		//
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		//
//		GNU General Public License for more details.				//
//											//
//		You should have received a copy of the GNU General Public License	//
//		along with this program; if not, write to the Free Software		//
//		Foundation, Inc., 51 Franklin Street, Fifth Floor, 			//
//		Boston, MA  02110-1301, USA						//
//											//
//////////////////////////////////////////////////////////////////////////////////////////


#include "crackerrors.h"

// this value will be returned on calling CrackGetDriverVersion()
#define	crack_VERSION		0.50

// device file used for communication between user- and kernel-space
#define crack_DEVICE 		"/dev/cracklinux"

// these values will be returned on executing modinfo
#define crack_AUTHOR		"Marko Euth <linux@euth.de>"
#define crack_LICENCE		"GPL"
#define crack_DESCRIPTION	"cracklinux"

// activate this for enable debugging messages.
#define crackDEBUG
//#define crackDEBUG2

// structure for data transfer between user- and kernel-space
typedef struct
{
	unsigned long physical_address;
	unsigned long *virtual_address;
	unsigned long value;
	unsigned long address;
	int irq;
	int status;
	int type;
	int priority;

} PARAMETER_IO;

// structure for data transfer at installing isr
typedef struct
{
	unsigned long interrupt;
	void (*function)(void * argument);
	void * parameter;
	
} PARAMETER_ISR;

// operation modes the module can perform
#define IO_READ		0x0010
#define IO_WRITE	0x0011
#define MEM_READ	0x0020
#define MEM_WRITE	0x0021
#define IRQ_REQUEST	0x0030
#define IRQ_FREE	0x0031
#define IRQ_CHECK	0x0032
#define IO_ENABLE	0x0040
#define IO_DISABLE	0x0041
#define SET_NICE	0x0050

// possible states of interrupts in kernel module
#define FREE		0
#define USED		1

// possible states of threads in the library
#define RUN		0
#define STOP		1

// possible states of Ports
#define PORT_UNLOCK	0
#define PORT_LOCK	1

// possible IO Types
#define crackBYTE	0
#define crackWORD	1
#define crackDWORD	2

//////////////////////////////////////////////////////////////////////////////////////////
//// EOF /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
