
//////////////////////////////////////////////////////////////////////////////////////////
//											//
// file:	libcracklinux.c								//
// version:	5.0									//
// author:	copyright (c) 2006 by Marko Euth <linux@euth.de>, www.euth.de		//
// last change: 09.03.2006								//
// description: This is the library libcracklinux.so which contents the functions	//
//		declared in libcracklinux.h and used by the application.		//
// 		It provides a simple interface for using the functionality of the	//
//		kernel module cracklinux.ko. For using, it has du be linked to the	//
//		application.								//
//											//
// license:	This program is free software; you can redistribute it and/or		//
//		modify it under the terms of the GNU General Public License		//
//		as published by the Free Software Foundation; either version 2		//
//		of the License, or (at your option) any later version.			//
//											//
//		This program is distributed in the hope that it will be useful,		//
//		but WITHOUT ANY WARRANTY; without even the implied warranty of		//
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		//
//		GNU General Public License for more details.				//
//											//
//		You should have received a copy of the GNU General Public License	//
//		along with this program; if not, write to the Free Software		//
//		Foundation, Inc., 51 Franklin Street, Fifth Floor, 			//
//		Boston, MA  02110-1301, USA						//
//											//
//////////////////////////////////////////////////////////////////////////////////////////



#include <stdio.h>		// printf()
#include <fcntl.h>		// open(), defines
#include <unistd.h>		// fork(), execlp(), close(), write(), read()
#include <sys/ioctl.h>		// ioctl()
#include <sys/wait.h>		// wait()
#include <pthread.h>		// pthread_th, pthread_create()
#include <errno.h>

#include "cracklinux.h"

// function for removing alle installed isr
int CrackRemoveAllInterruptCallbackRoutines(void);

// file descriptor used for device file crack_DEVICE
int fd;

// array of structures for transfering data to threads, one for each isr
PARAMETER_ISR thread_data[32];
// array of threads, one for each isr
pthread_t th[32];
// array of states, one for each isr
int stopThread[32];

//////////////////////////////////////////////////////////////////////////////////////////
//// control and transfer functions //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// open devicefile
int CrackOpen(void)
{
	// open device file in read/write mode
	fd = open( crack_DEVICE, O_RDWR );
	if( fd < 0 ) { return -errno; }
}

// close devicefile
int CrackClose(void)
{
	int errCODE;

	// remove all installed isr before closing filedescriptor
	errCODE = CrackRemoveAllInterruptCallbackRoutines();

	// close filedescriptor
	close(fd);

	return errCODE;
}

// perform operation
int CrackSetMode(unsigned int mode, PARAMETER_IO * transfer)
{
	int ret_val;
	unsigned long * struct_pointer = (unsigned long *) transfer;

	// send ioctl command and pointer to module
	ret_val = ioctl(fd, mode, struct_pointer);
	if (ret_val != 0){ return -errno; }

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// driver version //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// returns version
float CrackGetDriverVersion(void)
{
	return crack_VERSION;
}


//////////////////////////////////////////////////////////////////////////////////////////
//// load and unload module //////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// loads module
int CrackLoad(void)
{
	int pid, ret_pid;
	int status;

	// create child process
	if( (pid=fork()) == 0 )
	{		
		// insmod is executed in childprocess
		if( execlp("/usr/sbin/crackIT","/usr/sbin/crackIT","start", NULL) != 0 )
			{ return -cERR_EXEC; }
	}
	else
	{
		// return cERR_FORK when fork failed
		if( pid < 0 ){ return -cERR_FORK; }
		else
		{
			// parent process waits on childprocess
			if( ( ret_pid = wait(&status) ) != pid ) { return -cERR_WAIT; }

			// return cERR_COMMAND when insmod failed
			if(status != 0) return -cERR_COMMAND;
		}
	 }

	return 0;
}

// unloads module
int CrackUnload(void)
{
	int pid, ret_pid;
	int status;

	// create child process
	if( (pid=fork()) == 0 )
	{
		// rmmod is executed in childprocess
		if( execlp("/usr/sbin/crackIT","/usr/sbin/crackIT","stop", NULL) != 0 )
			{ return -cERR_EXEC; }
	}
	else
	{
		// return cERR_FORK when fork failed
		if( pid < 0 ){ return -cERR_FORK; }
		else
		{
			// parent process waits on childprocess
			if( ( ret_pid = wait(&status) ) != pid ) { return -cERR_WAIT; }

			// return cERR_COMMAND when rmmod failed
			if(status != 0) return -cERR_COMMAND;
		}
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// read and write ports ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

unsigned char CrackReadIoByte(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackBYTE;

	// set kernel module to IO_READ mode
	*errCODE = CrackSetMode(IO_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return value to application
	return (unsigned char) parameter_user.value;
}

unsigned short CrackReadIoWord(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackWORD;

	// set kernel module to IO_READ mode
	*errCODE = CrackSetMode(IO_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return value to application
	return (unsigned short) parameter_user.value;
}

unsigned long CrackReadIoDword(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackDWORD;

	// set kernel module to IO_READ mode
	*errCODE = CrackSetMode(IO_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return value to application
	return (unsigned long) parameter_user.value;
}

int CrackWriteIoByte(unsigned long address, unsigned char value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = (unsigned long) value;
	parameter_user.type = crackBYTE;

	// set kernel module to IO_WRITE mode
	errCODE = CrackSetMode(IO_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

int CrackWriteIoWord(unsigned long address, unsigned short value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = (unsigned long) value;
	parameter_user.type = crackWORD;

	// set kernel module to IO_WRITE mode
	errCODE = CrackSetMode(IO_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

int CrackWriteIoDword(unsigned long address, unsigned long value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write port address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = value;
	parameter_user.type = crackDWORD;

	// set kernel module to IO_WRITE mode
	errCODE = CrackSetMode(IO_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// Read and Write physical Memory //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

unsigned char CrackReadPhysicalMemoryByte(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackBYTE;

	// set kernel module to MEM_READ mode
	*errCODE = CrackSetMode(MEM_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return read value
	return (unsigned char) parameter_user.value;
}

unsigned short CrackReadPhysicalMemoryWord(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackWORD;

	// set kernel module to MEM_READ mode
	*errCODE = CrackSetMode(MEM_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return read value
	return (unsigned short) parameter_user.value;
}

unsigned long CrackReadPhysicalMemoryDword(unsigned long address, int * errCODE)
{
	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address and type to structure
	parameter_user.physical_address = address;
	parameter_user.type = crackDWORD;

	// set kernel module to MEM_READ mode
	*errCODE = CrackSetMode(MEM_READ, &parameter_user);
		if (*errCODE != 0){ return 0; }

	// return read value
	return parameter_user.value;
}

int CrackWritePhysicalMemoryByte(unsigned long address, unsigned char value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = (unsigned long) value;
	parameter_user.type = crackBYTE;

	// set kernel module MEM_WRITE mode
	errCODE = CrackSetMode(MEM_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

int CrackWritePhysicalMemoryWord(unsigned long address, unsigned short value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = (unsigned long) value;
	parameter_user.type = crackWORD;

	// set kernel module MEM_WRITE mode
	errCODE = CrackSetMode(MEM_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

int CrackWritePhysicalMemoryDword(unsigned long address, unsigned long value)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write memory address, data and type to structure
	parameter_user.physical_address = address;
	parameter_user.value = value;
	parameter_user.type = crackDWORD;

	// set kernel module MEM_WRITE mode
	errCODE = CrackSetMode(MEM_WRITE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// install and uninstall ISR ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// check whether an isr is installed or not for an given irq
int CrackCheckISR(int interrupt)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write interrupt number to structure
	parameter_user.irq = interrupt;

	// set kernel module to IRQ_CHECK mode
	errCODE = CrackSetMode(IRQ_CHECK, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	// return status
	return parameter_user.status;
}

// thread that is running as ISR in user space
void crack_THREAD( void * irq )
{
	int errCODE;
	int interrupt = (int) irq;
	int writeBytes;

	// copy data from transfer structure to local parameters for each thread
		// interrupt number
		unsigned long thread_interrupt = thread_data[interrupt].interrupt;
		// function pointer
		void (*userspaceISR)(void * argument) = thread_data[interrupt].function;
		// pointer on argument	
		void * thread_argument = thread_data[interrupt].parameter;

	while(1){
		// transfer irq number to kernel module, 
		// waits here for trigger from kernel mode isr
		writeBytes = write( fd, &interrupt, sizeof(interrupt) );
		if (writeBytes != 0){ return; }

		// exit thread when isr is uninstalled during waiting on interrupt
		if (stopThread[thread_interrupt] == STOP){ return; }

		// call user ISR and forward argument pointer
		userspaceISR(thread_argument);

		// exit thread when irq is freed during user isr is running
		if (stopThread[thread_interrupt] == STOP){ return; }
	}
}

// installs ISR and starts thread
int CrackInstallInterruptCallbackRoutine \
			(int interrupt, void (*isr)(void * arg), void * argument)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write interrupt number to structure
	parameter_user.irq = interrupt;

	// write irq, function pointer and argument pointer to thread structure
	thread_data[interrupt].interrupt = interrupt;
	thread_data[interrupt].function = isr;
	thread_data[interrupt].parameter = argument;

	// reset exit signale for user thread
	stopThread[interrupt] = RUN;

	// transfer irq number and request irq in kernel mode, install kernel isr
	errCODE = CrackSetMode(IRQ_REQUEST, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	// start thread and give thread structure to it
	if(pthread_create(&th[interrupt], NULL, (void *)crack_THREAD, (void *)interrupt) != 0)
		{ return -cERR_THREADCREATE; }

	return 0;
}

// uninstalls ISR
int CrackRemoveInterruptCallbackRoutine(int interrupt)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write interrupt number to structure
	parameter_user.irq = interrupt;

	// test whether isr is installed or not
	errCODE = CrackCheckISR(interrupt);
		if (errCODE == 0){ return -cERR_NOTINSTALLED; }

	// set signal to exit user thread
	stopThread[interrupt] = STOP;

	// free irq in kernel mode
	errCODE = CrackSetMode(IRQ_FREE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	// wait here for ending thread
	errCODE = pthread_join( th[interrupt], NULL );
		if (errCODE != 0){ return -cERR_THREADJOIN; }

	return 0;
}

// uninstall all previously installed isr
int CrackRemoveAllInterruptCallbackRoutines()
{
	int errCODE;
	int counter;

	// call isr remove function with each possible interrupt number
	for ( counter=0; counter < 32; counter++ )
	{
		errCODE = CrackRemoveInterruptCallbackRoutine(counter);
			if (errCODE != 0){
				if (errCODE != -cERR_NOTINSTALLED) {return errCODE; }
			}
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// enable or disabel direct IO access //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

int CrackDirectPortIo(unsigned long address, int new_state)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// write address to structure
	parameter_user.address = address;

	// unlock port
	if (new_state == PORT_UNLOCK)
	{
		errCODE = CrackSetMode(IO_ENABLE, &parameter_user);
			if (errCODE != 0){ return errCODE; }
	}

	// lock port
	if (new_state == PORT_LOCK)
	{
		errCODE = CrackSetMode(IO_DISABLE, &parameter_user);
			if (errCODE != 0){ return errCODE; }
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// set process priority (NICE Level) ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

int CrackSetNice(int new_priority)
{
	int errCODE;

	// structure for data transfer between library and kernel module
	PARAMETER_IO parameter_user;

	// wirte new priority to structure
	parameter_user.priority = new_priority;

	// set new NICE level
	errCODE = CrackSetMode(SET_NICE, &parameter_user);
		if (errCODE != 0){ return errCODE; }

	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//// EOF /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
