
//////////////////////////////////////////////////////////////////////////////////////////
//											//
// file:	libcracklinux.h								//
// version:	5.0									//
// author:	copyright (c) 2006 by Marko Euth <linux@euth.de>, www.euth.de		//
// last change: 09.03.2006								//
// description: This is the Header File to the shared object libcracklinux.so.		//
//		It has to be included in application, when using cracklinux.		//
//		It contents declarations of the following functions:			//
//											//
//		CrackLoad()								//
//		CrackUnload()								//
//		CrackGetDriverVersion()							//
// 		CrackReadIo...()							//
//		CrackWriteIo...()							//
//		CrackReadPhysicalMemory()						//
//		CrackWritePhysicalMemory()						//
//		CrackInstallInterruptCallbackRoutine()					//
//		CrackRemoveInterruptCallbackRoutine()					//
//		CrackRemoveAllInterruptCallbackRoutines()				//
//		CrackSetNice()								//
//											//
// license:	This program is free software; you can redistribute it and/or		//
//		modify it under the terms of the GNU General Public License		//
//		as published by the Free Software Foundation; either version 2		//
//		of the License, or (at your option) any later version.			//
//											//
//		This program is distributed in the hope that it will be useful,		//
//		but WITHOUT ANY WARRANTY; without even the implied warranty of		//
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		//
//		GNU General Public License for more details.				//
//											//
//		You should have received a copy of the GNU General Public License	//
//		along with this program; if not, write to the Free Software		//
//		Foundation, Inc., 51 Franklin Street, Fifth Floor, 			//
//		Boston, MA  02110-1301, USA						//
//											//
//////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////
//											//
//   All possible Values that could be returned in case of an error are described in	//
//											//
//				     crackerrors.h					//
//											//
//////////////////////////////////////////////////////////////////////////////////////////

#include <crackerrors.h>

//////////////////////////////////////////////////////////////////////////////////////////
//// load and unload module //////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// loads kernel modul cracklinux.ko
int CrackLoad(void);

// unloads kernel modul cracklinux.ko
int CrackUnload(void);

//////////////////////////////////////////////////////////////////////////////////////////
//// open and close device file //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// open device file before first usage
int CrackOpen(void);

// close device file afer last usage
int CrackClose(void);

//////////////////////////////////////////////////////////////////////////////////////////
//// driver version //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// returns driver version
float CrackGetDriverVersion(void);
//
// return value:
//	any	driver version

//////////////////////////////////////////////////////////////////////////////////////////
//// read and write Ports ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// read from port
unsigned char CrackReadIoByte(unsigned long, int *);
unsigned short CrackReadIoWord(unsigned long, int *);
unsigned long CrackReadIoDword(unsigned long, int *);
//
// parameter:
//	first	address to read from
//	second	int pointer, used for error code
// return value:
//	any	read value	

// write to port
int CrackWriteIoByte(unsigned long, unsigned char);
int CrackWriteIoWord(unsigned long, unsigned short);
int CrackWriteIoDword(unsigned long, unsigned long);
//
// parameter:
//	first	address to write to
//	second	value to write

//////////////////////////////////////////////////////////////////////////////////////////
//// read and write memory ///////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// read from memory
unsigned char CrackReadPhysicalMemoryByte(unsigned long, int *);
unsigned short CrackReadPhysicalMemoryWord(unsigned long, int *);
unsigned long CrackReadPhysicalMemoryDword(unsigned long, int *);
//
// parameter:
//	first	address to read from
//	second	int pointer, used for error code
// Return Value:
//	any	read value	

// write to memory
int CrackWritePhysicalMemoryByte(unsigned long, unsigned char);
int CrackWritePhysicalMemoryWord(unsigned long, unsigned short);
int CrackWritePhysicalMemoryDword(unsigned long, unsigned long);
//
// parameter:
//	first	address to write to
//	second	value to write

//////////////////////////////////////////////////////////////////////////////////////////
//// install and deinstall user space ISR ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

// install isr
int CrackInstallInterruptCallbackRoutine(int, void (*func)(void *), void *);
//
// parameter:
//	first	number of interrupt to install
//	second	pointer on function to install as user space isr
//	third	pointer on argument of user space isr

// remove isr
int CrackRemoveInterruptCallbackRoutine(int);
//
// parameter:
//	first	number of interrupt to free

// remove all isr that has been installed with cracklinux
int CrackRemoveAllInterruptCallbackRoutines(void);

//////////////////////////////////////////////////////////////////////////////////////////
//// enable and disable direct port access ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

int CrackDirectPortIo(unsigned long, int);
//
// parameter:
//	first	address of port to change
//	second	new state this port should take

//////////////////////////////////////////////////////////////////////////////////////////
//// Set Process Priority (NICE Level) ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

int CrackSetNice(int);
//
// parameter:
//	first	new priority level

//////////////////////////////////////////////////////////////////////////////////////////
//// EOF /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
